"""
- author:Auorui(夏天是冰红茶)
- creation time:2022.10
- pyzjr is the Codebase accumulated by my python programming.
  At present, it is only for my personal use. If you want to
  use it, please contact me. Here are my email and WeChat.
- WeChat: z15583909992
- Email: zjricetea@gmail.com
- Note: Currently still being updated, please refer to the latest version for any changes that may occur
"""

__version__ = "1.1.3.1"

from pyzjr.Enimage import Filter,Enhance,Random_Enhance,Retinex
from pyzjr.ColorModule import *
from pyzjr.utils import *
from pyzjr.video import *
from pyzjr.Z import * 
from pyzjr.zmath import *
from pyzjr.PIC import *
from pyzjr.FM import *
from pyzjr.Showimage import *
from pyzjr.dlearn.backbone import *
from pyzjr.SeniorVision import *

